#include <iostream>
#include <fstream>
#include <chrono>
#include "lib/nBodySim.hpp"

int main(int argc, char** argv) {
    std::string datafile;

    switch (argc) {
        case 1:
            datafile = "../data/data.ascii";
            break;
        case 2:
            datafile = argv[1];
            break;
        default:
            std::cerr << "Usage: " << argv[0] << " [datafile]" << std::endl;
            return 1;
    }

    double dt = 0.0001;
    unsigned nSteps = 200;

    nBodySim sim(datafile);
    // double* positions = sim.getPositions();
    unsigned N = sim.getNParticles();
    std::cout << "N = " << N << std::endl;

    try {
        auto start = std::chrono::high_resolution_clock::now();
        sim.runSimulation(dt, nSteps);
        auto stop = std::chrono::high_resolution_clock::now();

        auto duration = std::chrono::duration_cast<std::chrono::microseconds>(stop - start);
        std::cout << "Computation Time: " << duration.count()/1000000.0 << " seconds" << std::endl;
    } catch (std::exception& e) {
        std::cerr << e.what() << std::endl;
        // save state of simulation
        std::ofstream errfile("../out/errstate.txt");
        sim.saveState2file(nSteps, errfile);
        errfile.close();
        return 1;
    }
    
    // save state of simulation
    std::ofstream endfile("../out/endstate.txt");
    sim.saveState2file(nSteps, endfile);
    endfile.close();

    return 0;
}